@@ -5,6 +5,7 @@ import os.path
 import re
 from functools import reduce
 from io import StringIO
+import html
 
 import prettytable
 import sqlalchemy
@@ -19,6 +20,7 @@ except ImportError:
 
 from sql.telemetry import telemetry
 import logging
+import warnings
 
 
 def unduplicate_field_names(field_names):
@@ -107,14 +109,17 @@ class ResultSet(list, ColumnGuesserMixin):
         self.keys = {}
         if sqlaproxy.returns_rows:
             self.keys = sqlaproxy.keys()
-            if config.autolimit:
+            if isinstance(config.autolimit, int) and config.autolimit > 0:
                 list.__init__(self, sqlaproxy.fetchmany(size=config.autolimit))
             else:
                 list.__init__(self, sqlaproxy.fetchall())
             self.field_names = unduplicate_field_names(self.keys)
-            self.pretty = PrettyTable(
-                self.field_names, style=prettytable.__dict__[config.style.upper()]
-            )
+
+            _style = None
+            if isinstance(config.style, str):
+                _style = prettytable.__dict__[config.style.upper()]
+
+            self.pretty = PrettyTable(self.field_names, style=_style)
         else:
             list.__init__(self, [])
             self.pretty = None
@@ -124,6 +129,8 @@ class ResultSet(list, ColumnGuesserMixin):
         if self.pretty:
             self.pretty.add_rows(self)
             result = self.pretty.get_html_string()
+            # to create clickable links
+            result = html.unescape(result)
             result = _cell_with_spaces_pattern.sub(_nonbreaking_spaces, result)
             if self.config.displaylimit and len(self) > self.config.displaylimit:
                 HTML = (
@@ -173,15 +180,17 @@ class ResultSet(list, ColumnGuesserMixin):
         frame = pd.DataFrame(self, columns=(self and self.keys) or [])
         payload[
             "connection_info"
-        ] = sql.connection.Connection._get_curr_connection_info()
+        ] = sql.connection.Connection._get_curr_sqlalchemy_connection_info()
         return frame
 
     @telemetry.log_call("polars-data-frame")
-    def PolarsDataFrame(self):
+    def PolarsDataFrame(self, **polars_dataframe_kwargs):
         "Returns a Polars DataFrame instance built from the result set."
         import polars as pl
 
-        frame = pl.DataFrame((tuple(row) for row in self), schema=self.keys)
+        frame = pl.DataFrame(
+            (tuple(row) for row in self), schema=self.keys, **polars_dataframe_kwargs
+        )
         return frame
 
     @telemetry.log_call("pie")
@@ -386,53 +395,90 @@ def _commit(conn, config, manual_commit):
             print("The database does not support the COMMIT command")
 
 
-def run(conn, sql, config, user_namespace):
-    if sql.strip():
-        for statement in sqlparse.split(sql):
-            first_word = sql.strip().split()[0].lower()
-            manual_commit = False
-            if first_word == "begin":
-                raise Exception("ipython_sql does not support transactions")
-            if first_word.startswith("\\") and (
-                "postgres" in str(conn.dialect) or "redshift" in str(conn.dialect)
-            ):
-                if not PGSpecial:
-                    raise ImportError("pgspecial not installed")
-                pgspecial = PGSpecial()
-                _, cur, headers, _ = pgspecial.execute(
-                    conn.session.connection.cursor(), statement
-                )[0]
-                result = FakeResultProxy(cur, headers)
-            else:
-                txt = sqlalchemy.sql.text(statement)
-                if config.autocommit:
-                    try:
-                        conn.session.execution_options(isolation_level="AUTOCOMMIT")
-                    except Exception as e:
-                        logging.debug(
-                            f"The database driver doesn't support such "
-                            f"AUTOCOMMIT execution option"
-                            f"\nPerhaps you can try running a manual COMMIT command"
-                            f"\nMessage from the database driver\n\t"
-                            f"Exception:  {e}\n",  # noqa: F841
-                        )
-                        manual_commit = True
-                result = conn.session.execute(txt, user_namespace)
-            _commit(conn=conn, config=config, manual_commit=manual_commit)
-            if result and config.feedback:
-                print(interpret_rowcount(result.rowcount))
+def is_postgres_or_redshift(dialect):
+    """Checks if dialect is postgres or redshift"""
+    return "postgres" in str(dialect) or "redshift" in str(dialect)
 
-        resultset = ResultSet(result, config)
-        if config.autopandas:
-            return resultset.DataFrame()
-        elif config.autopolars:
-            return resultset.PolarsDataFrame()
-        else:
-            return resultset
-        # returning only last result, intentionally
+
+def is_pytds(dialect):
+    """Checks if driver is pytds"""
+    return "pytds" in str(dialect)
+
+
+def handle_postgres_special(conn, statement):
+    """Execute a PostgreSQL special statement using PGSpecial module."""
+    if not PGSpecial:
+        raise ImportError("pgspecial not installed")
+    pgspecial = PGSpecial()
+    _, cur, headers, _ = pgspecial.execute(conn.session.connection.cursor(), statement)[
+        0
+    ]
+    return FakeResultProxy(cur, headers)
+
+
+def set_autocommit(conn, config):
+    """Sets the autocommit setting for a database connection."""
+    if is_pytds(conn.dialect):
+        warnings.warn(
+            "Autocommit is not supported for pytds, thus is automatically disabled"
+        )
+        return False
+    if config.autocommit:
+        try:
+            conn.session.execution_options(isolation_level="AUTOCOMMIT")
+        except Exception as e:
+            logging.debug(
+                f"The database driver doesn't support such "
+                f"AUTOCOMMIT execution option"
+                f"\nPerhaps you can try running a manual COMMIT command"
+                f"\nMessage from the database driver\n\t"
+                f"Exception:  {e}\n",  # noqa: F841
+            )
+            return True
+    return False
+
+
+def select_df_type(resultset, config):
+    """
+    Converts the input resultset to either a Pandas DataFrame
+    or Polars DataFrame based on the config settings.
+    """
+    if config.autopandas:
+        return resultset.DataFrame()
+    elif config.autopolars:
+        return resultset.PolarsDataFrame(**config.polars_dataframe_kwargs)
     else:
+        return resultset
+    # returning only last result, intentionally
+
+
+def run(conn, sql, config):
+    if not sql.strip():
+        # returning only when sql is empty string
         return "Connected: %s" % conn.name
 
+    for statement in sqlparse.split(sql):
+        first_word = sql.strip().split()[0].lower()
+        manual_commit = False
+        if first_word == "begin":
+            raise ValueError("ipython_sql does not support transactions")
+        if first_word.startswith("\\") and is_postgres_or_redshift(conn.dialect):
+            result = handle_postgres_special(conn, statement)
+        else:
+            txt = sqlalchemy.sql.text(statement)
+            manual_commit = set_autocommit(conn, config)
+            result = conn.session.execute(txt)
+        _commit(conn=conn, config=config, manual_commit=manual_commit)
+        if result and config.feedback:
+            print(interpret_rowcount(result.rowcount))
+
+    resultset = ResultSet(result, config)
+    return select_df_type(resultset, config)
+
+
+def raw_run(conn, sql):
+    return conn.session.execute(sqlalchemy.sql.text(sql))
+
 
 class PrettyTable(prettytable.PrettyTable):
     def __init__(self, *args, **kwargs):
@@ -452,4 +498,10 @@ class PrettyTable(prettytable.PrettyTable):
         else:
             self.row_count = min(len(data), self.displaylimit)
         for row in data[: self.displaylimit]:
-            self.add_row(row)
+            formatted_row = []
+            for cell in row:
+                if isinstance(cell, str) and cell.startswith("http"):
+                    formatted_row.append("<a href={}>{}</a>".format(cell, cell))
+                else:
+                    formatted_row.append(cell)
+            self.add_row(formatted_row)
